#!/bin/bash
# ************************************************************************
# *                                                                      *
# *   Bash-shell program malo_check.sh checks the first and last         *
# *   dates of files with time series related to mass loading.           *
# *                                                                      *
# * ### 21-MAR-2013  malo_check.sh v3.6 (c)  L. Petrov  08-FEB-2021 ###  *
# *                                                                      *
# ************************************************************************
hostname=`uname -n` 
if [ $hostname == "astrogeo" ]; then
      export BIN_DIR=/opt64/bin
      export LIB_DIR=/opt64/lib
      host_prefix="astrogeo"
      imsl_dir=/massloading.net
      ners_dir=/earthrotation.net
      malo_query=/astrogeo.org/web_exec/malo_query.py
elif [ `uname -n` == "earthrotation" ] ; then
      export BIN_DIR=/opt64/bin
      export LIB_DIR=/opt64/lib
      host_prefix="astrogeo"
      imsl_dir=/massloading.net
      ners_dir=/earthrotation.net
      malo_query=/astrogeo.org/web_exec/malo_query.py
elif [ `uname -n` == "gs61a-geodev-a" ] ; then
      export BIN_DIR=/opt64/bin
      export LIB_DIR=/opt64/lib
      host_prefix="deva"
      export MALO_ROOT=`$BIN_DIR/malo_inq root`
      imsl_dir=/imsl/massloading.net
      ners_dir=/imsl/earthrotation.net
      malo_query=${MALO_ROOT}/web_script/malo_query.py
elif [ `uname -n` == "geopod" ] ; then
      export BIN_DIR=/Users/lpetrov/opt/bin
      export LIB_DIR=/Users/lpetrov/opt/lib
      host_prefix="geopod"
      export MALO_ROOT=`$BIN_DIR/malo_inq root`
      imsl_dir=/imsl/massloading.net
      ners_dir=/imsl/earthrotation.net
      malo_query=${MALO_ROOT}/web_script/malo_query.py
else
      echo "malo_check.sh: Unknown host "`uname -n`
fi
export GOMP_STACKSIZE=2000000
export MALO_SHARE=`$BIN_DIR/malo_inq  share`
export MALO_SCRIPT=`$BIN_DIR/malo_inq script`
export MALO_DIR=`$BIN_DIR/malo_inq root`
export MALO_BIN=`$BIN_DIR/malo_inq bin_static`
export SPD_SHARE="/opt64/spd/share"
#
export PGPLOT_FONT=$BIN_DIR/grfont.dat
export PGPLOT_XW_MARGIN=1.0
#
export imsl_plot=${imsl_dir}/plots
export ners_fcs=${ners_dir}/ners/eop.ners
export ners_ser=${ners_dir}/ners/ners_series.txt
export ners_plot=${ners_dir}/ners/plots
export tmp_ser=${ners_dir}/ners/ners_ser__$$
export ners_stat_conf=$MALO_SHARE/${host_prefix}_eop_stat.conf
if [ -f $MALO_SHARE/${host_prefix}_stat_all.conf ]; then
     export imsl_stat_conf=$MALO_SHARE/${host_prefix}_stat_all.conf
   else
     export imsl_stat_conf=$MALO_SHARE/${host_prefix}_stat.conf
fi
export ners_stat_file=`grep lat_file $ners_stat_conf | awk '{print $2}'`
export imsl_stat_file=`grep lat_file $imsl_stat_conf | awk '{print $2}'`
#
pid_ond=`ps -eaf | grep malo_ondemand | grep -v grep`
#
if [ $hostname == "earthrotation" ] ; then
      ond_host_prefix=$hostname 
else      
      ond_host_prefix=$host_prefix
fi
#
if [ "$pid_ond" == "" ]; then
     DATE_ISO=`date "+%Y.%m.%d_%H:%M:%S.%N"`
     echo "Launched malo_ondemand on $DATE_ISO" >> /imsl/ondemand/launch.log
     if [ $hostname == "astrogeo" ] || [ $hostname == "earthrotation" ] ; then
          $MALO_BIN/malo_ondemand $MALO_SHARE/${ond_host_prefix}_ond.conf 3 2>&1  >> /imsl/ondemand/ond.log &
     elif [ $hostname == "gs698-geopod.gsfc.nasa.gov" ]; then
           echo "malo_ondemand will not be launched" >> /dev/null
     elif [ `uname -n` == "gs61a-geodev-a" ] ; then
           echo "malo_ondemand will not be launched" >> /dev/null
     else
           echo "malo_check.sh: Unknown host $hostname"
	   exit 1
     fi
     if [ "$1" == "ond" ]; then
          exit 0
     fi
fi
if [ $hostname == "${host_prefix}" ] || [ $hostname == "earthrotation" ] || [ $hostname == "gs61a-geodev-a" ]; then
     cp /imsl/logs/${host_prefix}.lat /imsl/logs/${host_prefix}_mini.lat
     if [ -f $MALO_SHARE/${host_prefix}_stat_all.conf ]; then
          python3 $MALO_SCRIPT/malo_stat.py -c $MALO_SHARE/${host_prefix}_stat_all.conf -v 1
     fi
     python3 $MALO_SCRIPT/malo_stat.py -c $MALO_SHARE/${host_prefix}_stat.conf -v 1
else
     python3 $MALO_SCRIPT/malo_stat.py -c $MALO_SHARE/geopod_stat.conf -v 1
fi
#
$MALO_BIN/malo_latency_plot $imsl_stat_conf $imsl_stat_file $imsl_plot # >& /imsl/logs/imsl_latency_plots.log
if [ -f $MALO_SHARE/${host_prefix}_eop.conf ]; then
      $MALO_BIN/malo_latency_plot $ners_stat_conf $ners_stat_file $ners_plot # >& /imsl/logs/ners_latency_plots.log
      python3 $MALO_SCRIPT/get_eop.py -c $MALO_SHARE/${host_prefix}_eop.conf -l
      if [ $status == ]; then
           cat $ners_ser > $tmp_ser
           $MALO_BIN/show_eop_fcs $ners_fcs now ser >> $tmp_ser
           mv  $tmp_ser $ners_ser
      fi
fi
#################
#
###  ATM GEOSFPIT 
#
#################
$malo_query -s atm -m GEOSFPIT -a get_loading_first_date >\
             ${imsl_dir}/atm/dates/geosfpit_loading_first_date.txt__$$
mv           ${imsl_dir}/atm/dates/geosfpit_loading_first_date.txt__$$ \
             ${imsl_dir}/atm/dates/geosfpit_loading_first_date.txt
#
$malo_query -s atm -m GEOSFPIT -a get_loading_last_date >\
             ${imsl_dir}/atm/dates/geosfpit_loading_last_date.txt__$$
mv           ${imsl_dir}/atm/dates/geosfpit_loading_last_date.txt__$$ \
             ${imsl_dir}/atm/dates/geosfpit_loading_last_date.txt
#
$malo_query -s atm -m GEOSFPIT -a get_loading_last_update >\
             ${imsl_dir}/atm/dates/geosfpit_loading_last_update.txt__$$
mv           ${imsl_dir}/atm/dates/geosfpit_loading_last_update.txt__$$ \
             ${imsl_dir}/atm/dates/geosfpit_loading_last_update.txt
#
$malo_query -s atm -m GEOSFPIT -a get_vgep_first_date >\
             ${imsl_dir}/atm/dates/geosfpit_vgep_first_date.txt__$$
mv           ${imsl_dir}/atm/dates/geosfpit_vgep_first_date.txt__$$ \
             ${imsl_dir}/atm/dates/geosfpit_vgep_first_date.txt
#
$malo_query -s atm -m GEOSFPIT -a get_vgep_last_date >\
             ${imsl_dir}/atm/dates/geosfpit_vgep_last_date.txt__$$
mv           ${imsl_dir}/atm/dates/geosfpit_vgep_last_date.txt__$$ \
             ${imsl_dir}/atm/dates/geosfpit_vgep_last_date.txt
#
$malo_query -s atm -m GEOSFPIT -a get_vgep_last_update >\
             ${imsl_dir}/atm/dates/geosfpit_vgep_last_update.txt__$$
mv           ${imsl_dir}/atm/dates/geosfpit_vgep_last_update.txt__$$ \
             ${imsl_dir}/atm/dates/geosfpit_vgep_last_update.txt
#################
#
###  ATM MERRA2 
#
#################
$malo_query -s atm -m MERRA2 -a get_loading_first_date >\
             ${imsl_dir}/atm/dates/merra2_loading_first_date.txt__$$
mv           ${imsl_dir}/atm/dates/merra2_loading_first_date.txt__$$ \
             ${imsl_dir}/atm/dates/merra2_loading_first_date.txt
#
$malo_query -s atm -m MERRA2 -a get_loading_last_date >\
             ${imsl_dir}/atm/dates/merra2_loading_last_date.txt__$$
mv           ${imsl_dir}/atm/dates/merra2_loading_last_date.txt__$$ \
             ${imsl_dir}/atm/dates/merra2_loading_last_date.txt
#
$malo_query -s atm -m MERRA2 -a get_loading_last_update >\
             ${imsl_dir}/atm/dates/merra2_loading_last_update.txt__$$
mv           ${imsl_dir}/atm/dates/merra2_loading_last_update.txt__$$ \
             ${imsl_dir}/atm/dates/merra2_loading_last_update.txt
#
$malo_query -s atm -m MERRA2 -a get_vgep_first_date >\
             ${imsl_dir}/atm/dates/merra2_vgep_first_date.txt__$$
mv           ${imsl_dir}/atm/dates/merra2_vgep_first_date.txt__$$ \
             ${imsl_dir}/atm/dates/merra2_vgep_first_date.txt
#
$malo_query -s atm -m MERRA2 -a get_vgep_last_date >\
             ${imsl_dir}/atm/dates/merra2_vgep_last_date.txt__$$
mv           ${imsl_dir}/atm/dates/merra2_vgep_last_date.txt__$$ \
             ${imsl_dir}/atm/dates/merra2_vgep_last_date.txt
#
$malo_query -s atm -m MERRA2 -a get_vgep_last_update >\
             ${imsl_dir}/atm/dates/merra2_vgep_last_update.txt__$$
mv           ${imsl_dir}/atm/dates/merra2_vgep_last_update.txt__$$ \
             ${imsl_dir}/atm/dates/merra2_vgep_last_update.txt
#################
#
###  LWS MERRA2 
#
#################
$malo_query -s lws -m MERRA2 -a get_loading_first_date >\
             ${imsl_dir}/lws/dates/merra2_loading_first_date.txt__$$
mv           ${imsl_dir}/lws/dates/merra2_loading_first_date.txt__$$ \
             ${imsl_dir}/lws/dates/merra2_loading_first_date.txt
#
$malo_query -s lws -m MERRA2 -a get_loading_last_date >\
             ${imsl_dir}/lws/dates/merra2_loading_last_date.txt__$$
mv           ${imsl_dir}/lws/dates/merra2_loading_last_date.txt__$$ \
             ${imsl_dir}/lws/dates/merra2_loading_last_date.txt
#
$malo_query -s lws -m MERRA2 -a get_loading_last_update >\
             ${imsl_dir}/lws/dates/merra2_loading_last_update.txt__$$
mv           ${imsl_dir}/lws/dates/merra2_loading_last_update.txt__$$ \
             ${imsl_dir}/lws/dates/merra2_loading_last_update.txt
#
$malo_query -s lws -m MERRA2 -a get_vgep_first_date >\
             ${imsl_dir}/lws/dates/merra2_vgep_first_date.txt__$$
mv           ${imsl_dir}/lws/dates/merra2_vgep_first_date.txt__$$ \
             ${imsl_dir}/lws/dates/merra2_vgep_first_date.txt
#
$malo_query -s lws -m MERRA2 -a get_vgep_last_date >\
             ${imsl_dir}/lws/dates/merra2_vgep_last_date.txt__$$
mv           ${imsl_dir}/lws/dates/merra2_vgep_last_date.txt__$$ \
             ${imsl_dir}/lws/dates/merra2_vgep_last_date.txt
#
$malo_query -s lws -m MERRA2 -a get_vgep_last_update >\
             ${imsl_dir}/lws/dates/merra2_vgep_last_update.txt__$$
mv           ${imsl_dir}/lws/dates/merra2_vgep_last_update.txt__$$ \
             ${imsl_dir}/lws/dates/merra2_vgep_last_update.txt
#################
#
###  LWS GEOSFPIT 
#
#################
$malo_query -s lws -m GEOSFPIT -a get_loading_first_date >\
             ${imsl_dir}/lws/dates/geosfpit_loading_first_date.txt__$$
mv           ${imsl_dir}/lws/dates/geosfpit_loading_first_date.txt__$$ \
             ${imsl_dir}/lws/dates/geosfpit_loading_first_date.txt
#
$malo_query -s lws -m GEOSFPIT -a get_loading_last_date >\
             ${imsl_dir}/lws/dates/geosfpit_loading_last_date.txt__$$
mv           ${imsl_dir}/lws/dates/geosfpit_loading_last_date.txt__$$ \
             ${imsl_dir}/lws/dates/geosfpit_loading_last_date.txt
#
$malo_query -s lws -m GEOSFPIT -a get_loading_last_update >\
             ${imsl_dir}/lws/dates/geosfpit_loading_last_update.txt__$$
mv           ${imsl_dir}/lws/dates/geosfpit_loading_last_update.txt__$$ \
             ${imsl_dir}/lws/dates/geosfpit_loading_last_update.txt
#
$malo_query -s lws -m GEOSFPIT -a get_vgep_first_date >\
             ${imsl_dir}/lws/dates/geosfpit_vgep_first_date.txt__$$
mv           ${imsl_dir}/lws/dates/geosfpit_vgep_first_date.txt__$$ \
             ${imsl_dir}/lws/dates/geosfpit_vgep_first_date.txt
#
$malo_query -s lws -m GEOSFPIT -a get_vgep_last_date >\
             ${imsl_dir}/lws/dates/geosfpit_vgep_last_date.txt__$$
mv           ${imsl_dir}/lws/dates/geosfpit_vgep_last_date.txt__$$ \
             ${imsl_dir}/lws/dates/geosfpit_vgep_last_date.txt
#
$malo_query -s lws -m GEOSFPIT -a get_vgep_last_update >\
             ${imsl_dir}/lws/dates/geosfpit_vgep_last_update.txt__$$
mv           ${imsl_dir}/lws/dates/geosfpit_vgep_last_update.txt__$$ \
             ${imsl_dir}/lws/dates/geosfpit_vgep_last_update.txt
############
#
# NTO OMCT05
#
############
$malo_query -s nto -m OMCT05 -a get_loading_first_date >\
             ${imsl_dir}/nto/dates/omct05_loading_first_date.txt__$$
mv           ${imsl_dir}/nto/dates/omct05_loading_first_date.txt__$$ \
             ${imsl_dir}/nto/dates/omct05_loading_first_date.txt
#
$malo_query -s nto -m OMCT05 --a get_loading_last_date >\
             ${imsl_dir}/nto/dates/omct05_loading_last_date.txt__$$
mv           ${imsl_dir}/nto/dates/omct05_loading_last_date.txt__$$ \
             ${imsl_dir}/nto/dates/omct05_loading_last_date.txt
#
$malo_query -s nto -m OMCT05 --a get_loading_last_update >\
             ${imsl_dir}/nto/dates/omct05_loading_last_update.txt__$$
mv           ${imsl_dir}/nto/dates/omct05_loading_last_update.txt__$$ \
             ${imsl_dir}/nto/dates/omct05_loading_last_update.txt
#
$malo_query -s nto -m OMCT05 --a get_vgep_first_date >\
             ${imsl_dir}/nto/dates/omct05_vgep_first_date.txt__$$
mv           ${imsl_dir}/nto/dates/omct05_vgep_first_date.txt__$$ \
             ${imsl_dir}/nto/dates/omct05_vgep_first_date.txt
#
$malo_query -s nto -m OMCT05 --a get_vgep_last_date >\
             ${imsl_dir}/nto/dates/omct05_vgep_last_date.txt__$$
mv           ${imsl_dir}/nto/dates/omct05_vgep_last_date.txt__$$ \
             ${imsl_dir}/nto/dates/omct05_vgep_last_date.txt
#
$malo_query -s nto -m OMCT05 --a get_vgep_last_update >\
             ${imsl_dir}/nto/dates/omct05_vgep_last_update.txt__$$
mv           ${imsl_dir}/nto/dates/omct05_vgep_last_update.txt__$$ \
             ${imsl_dir}/nto/dates/omct05_vgep_last_update.txt
############
#
# NTO MPIOM06
#
############
$malo_query -s nto -m MPIOM06 -a get_loading_first_date >\
             ${imsl_dir}/nto/dates/mpiom06_loading_first_date.txt__$$
mv           ${imsl_dir}/nto/dates/mpiom06_loading_first_date.txt__$$ \
             ${imsl_dir}/nto/dates/mpiom06_loading_first_date.txt
#
$malo_query -s nto -m MPIOM06 --a get_loading_last_date >\
             ${imsl_dir}/nto/dates/mpiom06_loading_last_date.txt__$$
mv           ${imsl_dir}/nto/dates/mpiom06_loading_last_date.txt__$$ \
             ${imsl_dir}/nto/dates/mpiom06_loading_last_date.txt
#
$malo_query -s nto -m MPIOM06 --a get_loading_last_update >\
             ${imsl_dir}/nto/dates/mpiom06_loading_last_update.txt__$$
mv           ${imsl_dir}/nto/dates/mpiom06_loading_last_update.txt__$$ \
             ${imsl_dir}/nto/dates/mpiom06_loading_last_update.txt
#
$malo_query -s nto -m MPIOM06 --a get_vgep_first_date >\
             ${imsl_dir}/nto/dates/mpiom06_vgep_first_date.txt__$$
mv           ${imsl_dir}/nto/dates/mpiom06_vgep_first_date.txt__$$ \
             ${imsl_dir}/nto/dates/mpiom06_vgep_first_date.txt
#
$malo_query -s nto -m MPIOM06 --a get_vgep_last_date >\
             ${imsl_dir}/nto/dates/mpiom06_vgep_last_date.txt__$$
mv           ${imsl_dir}/nto/dates/mpiom06_vgep_last_date.txt__$$ \
             ${imsl_dir}/nto/dates/mpiom06_vgep_last_date.txt
#
$malo_query -s nto -m MPIOM06 --a get_vgep_last_update >\
             ${imsl_dir}/nto/dates/mpiom06_vgep_last_update.txt__$$
mv           ${imsl_dir}/nto/dates/mpiom06_vgep_last_update.txt__$$ \
             ${imsl_dir}/nto/dates/mpiom06_vgep_last_update.txt
#################
#
#  AAM
#
#################
$malo_query -s aam -m GEOSFCS -a get_aam_last_update >\
             $ners_dir/aam/dates/last_update.txt__$$
$malo_query -s aam -m GEOSFCS -a get_aam_last_date >\
             $ners_dir/aam/dates/last_date.txt__$$
$malo_query -s aam -m GEOSFCS -a get_aam_first_date >\
             $ners_dir/aam/dates/first_date.txt__$$
#
mv ${ners_dir}/aam/dates/last_update.txt__$$ ${ners_dir}/aam/dates/aam_geosfcs_last_update.txt
mv ${ners_dir}/aam/dates/last_date.txt__$$   ${ners_dir}/aam/dates/aam_geosfcs_last_date.txt
mv ${ners_dir}/aam/dates/first_date.txt__$$  ${ners_dir}/aam/dates/aam_geosfcs_first_date.txt
#
$malo_query -s aam -m GEOSFP -a get_aam_last_update >\
             ${ners_dir}/aam/dates/last_update.txt__$$
$malo_query -s aam -m GEOSFP -a get_aam_last_date >\
             ${ners_dir}/aam/dates/last_date.txt__$$
$malo_query -s aam -m GEOSFP -a get_aam_first_date >\
             ${ners_dir}/aam/dates/first_date.txt__$$
#
mv ${ners_dir}/aam/dates/last_update.txt__$$ ${ners_dir}/aam/dates/aam_geosfp_last_update.txt
mv ${ners_dir}/aam/dates/last_date.txt__$$   ${ners_dir}/aam/dates/aam_geosfp_last_date.txt
mv ${ners_dir}/aam/dates/first_date.txt__$$  ${ners_dir}/aam/dates/aam_geosfp_first_date.txt
#
$malo_query -s aam -m GEOSFPIT -a get_aam_last_update >\
             ${ners_dir}/aam/dates/last_update.txt__$$
$malo_query -s aam -m GEOSFPIT -a get_aam_last_date >\
             ${ners_dir}/aam/dates/last_date.txt__$$
$malo_query -s aam -m GEOSFPIT -a get_aam_first_date >\
             ${ners_dir}/aam/dates/first_date.txt__$$
#
mv ${ners_dir}/aam/dates/last_update.txt__$$ ${ners_dir}/aam/dates/aam_geosfpit_last_update.txt
mv ${ners_dir}/aam/dates/last_date.txt__$$   ${ners_dir}/aam/dates/aam_geosfpit_last_date.txt
mv ${ners_dir}/aam/dates/first_date.txt__$$  ${ners_dir}/aam/dates/aam_geosfpit_first_date.txt
#
$malo_query -s aam -m MERRA2 -a get_aam_last_update >\
             ${ners_dir}/aam/dates/last_update.txt__$$
$malo_query -s aam -m MERRA2 -a get_aam_last_date >\
             ${ners_dir}/aam/dates/last_date.txt__$$
$malo_query -s aam -m MERRA2 -a get_aam_first_date >\
             ${ners_dir}/aam/dates/first_date.txt__$$
#
mv ${ners_dir}/aam/dates/last_update.txt__$$ ${ners_dir}/aam/dates/aam_merra2_last_update.txt
mv ${ners_dir}/aam/dates/last_date.txt__$$   ${ners_dir}/aam/dates/aam_merra2_last_date.txt
mv ${ners_dir}/aam/dates/first_date.txt__$$  ${ners_dir}/aam/dates/aam_merra2_first_date.txt
#################
#
#  SPD merra2
#
#################
if [ $hostname == "astrogeo" ] || [ $hostname == "earthrotation" ]; then
   $malo_query -s spd -m MERRA -a get_spd_last_update >\
                /spd/dates/last_update.txt__$$
   $malo_query -s spd -m MERRA -a get_spd_last_date >\
                /spd/dates/last_date.txt__$$
   $malo_query -s spd -m MERRA -a get_spd_first_date >\
                /spd/dates/first_date.txt__$$
#
   mv /spd/dates/last_update.txt__$$ /spd/dates/spd_merra_last_update.txt
   mv /spd/dates/last_date.txt__$$   /spd/dates/spd_merra_last_date.txt
   mv /spd/dates/first_date.txt__$$  /spd/dates/spd_merra_first_date.txt
#
#   $malo_query -s spd -m MERRA2 -a get_spd_last_update >\
#                /spd/dates/last_update.txt__$$
#   $malo_query -s spd -m MERRA2 -a get_spd_last_date >\
#                /spd/dates/last_date.txt__$$
#   $malo_query -s spd -m MERRA2 -a get_spd_first_date >\
#                /spd/dates/first_date.txt__$$
##
#   mv /spd/dates/last_update.txt__$$ /spd/dates/spd_merra2_last_update.txt
#   mv /spd/dates/last_date.txt__$$   /spd/dates/spd_merra2_last_date.txt
#   mv /spd/dates/first_date.txt__$$  /spd/dates/spd_merra2_first_date.txt
#
   $malo_query -s spd -m GEOSFPIT -a get_spd_last_update >\
                /spd/dates/last_update.txt__$$
   $malo_query -s spd -m GEOSFPIT -a get_spd_last_date >\
                /spd/dates/last_date.txt__$$
   $malo_query -s spd -m GEOSFPIT -a get_spd_first_date >\
                /spd/dates/first_date.txt__$$
#
   mv /spd/dates/last_update.txt__$$ /spd/dates/spd_geosfpit_last_update.txt
   mv /spd/dates/last_date.txt__$$   /spd/dates/spd_geosfpit_last_date.txt
   mv /spd/dates/first_date.txt__$$  /spd/dates/spd_geosfpit_first_date.txt
#
   $malo_query -s aam -m MERRA2 -a get_aam_last_update >\
                ${ners_dir}/aam/dates/last_update.txt__$$
   $malo_query -s aam -m MERRA2 -a get_aam_last_date >\
                ${ners_dir}/aam/dates/last_date.txt__$$
   $malo_query -s aam -m MERRA2 -a get_aam_first_date >\
                ${ners_dir}/aam/dates/first_date.txt__$$
#
   mv ${ners_dir}/aam/dates/last_update.txt__$$ ${ners_dir}/aam/dates/aam_merra2_last_update.txt
   mv ${ners_dir}/aam/dates/last_date.txt__$$   ${ners_dir}/aam/dates/aam_merra2_last_date.txt
   mv ${ners_dir}/aam/dates/first_date.txt__$$  ${ners_dir}/aam/dates/aam_merra2_first_date.txt
#
   $malo_query -s spd -m MERRA2 -a get_spd_last_update >\
                /spd/dates/last_update.txt__$$
   $malo_query -s spd -m MERRA2 -a get_spd_last_date >\
                /spd/dates/last_date.txt__$$
   $malo_query -s spd -m MERRA2 -a get_spd_first_date >\
                /spd/dates/first_date.txt__$$
#
   mv /spd/dates/last_update.txt__$$ /spd/dates/spd_merra2_last_update.txt
   mv /spd/dates/last_date.txt__$$   /spd/dates/spd_merra2_last_date.txt
   mv /spd/dates/first_date.txt__$$  /spd/dates/spd_merra2_first_date.txt
#
   $malo_query -s spd -m GEOSFPIT -a get_spd_last_update >\
                /spd/dates/last_update.txt__$$
   $malo_query -s spd -m GEOSFPIT -a get_spd_last_date >\
                /spd/dates/last_date.txt__$$
   $malo_query -s spd -m GEOSFPIT -a get_spd_first_date >\
                /spd/dates/first_date.txt__$$
#
   mv /spd/dates/last_update.txt__$$ /spd/dates/spd_geosfpit_last_update.txt
   mv /spd/dates/last_date.txt__$$   /spd/dates/spd_geosfpit_last_date.txt
   mv /spd/dates/first_date.txt__$$  /spd/dates/spd_geosfpit_first_date.txt
fi
#
python3 $MALO_SCRIPT/malo_service.py \
       -c $MALO_SHARE/malo_stat_service.conf \
       -a get_num_files >& /imsl/logs/malo_num_files.txt
python3 $MALO_SCRIPT/malo_service.py \
       -c $SPD_SHARE/spd_stat_service.conf \
       -a get_num_files >& /imsl/logs/spd_num_files.txt
python3 $MALO_SCRIPT/malo_service.py \
       -c $MALO_SHARE/aam_stat_service.conf \
       -a get_num_files >& /imsl/logs/aam_num_files.txt
#
# --- Check for gaps and get statistics
#
$MALO_SCRIPT/imsl_check.csh > $imsl_dir/stat/${hostname}_imsl_check.txt
cat $imsl_dir/stat/${hostname}_imsl_check.txt | tail -1 | head -1 > $imsl_dir/stat/num_files.txt
cat $imsl_dir/stat/${hostname}_imsl_check.txt | tail -2 | head -1 > $imsl_dir/stat/num_grid_files.txt
cat $imsl_dir/stat/${hostname}_imsl_check.txt | tail -3 | head -1 > $imsl_dir/stat/num_grid_points.txt
